@@ -137,16 +137,14 @@ mod take_until_error {
             Ready({
                 if *this.is_terminated {
                     None
-                } else {
-                    if let Some(result) = ready!(this.stream.poll_next(cx)) {
-                        if result.is_err() {
-                            *this.is_terminated = true;
-                        }
-                        Some(result)
-                    } else {
+                } else if let Some(result) = ready!(this.stream.poll_next(cx)) {
+                    if result.is_err() {
                         *this.is_terminated = true;
-                        None
                     }
+                    Some(result)
+                } else {
+                    *this.is_terminated = true;
+                    None
                 }
             })
         }
@@ -245,7 +243,7 @@ mod try_stateful_map {
         fn poll_next(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Option<Self::Item>> {
             let mut this = self.project();
 
-            Poll::Ready(loop {
+            Poll::Ready({
                 if let Some(state) = this.state.take() {
                     match this.stream.as_mut().poll_next(cx) {
                         Ready(Some(Ok(in_item))) => {
@@ -254,23 +252,21 @@ mod try_stateful_map {
                             match result {
                                 Ok(Some((state, out_item))) => {
                                     *this.state = Some(state);
-                                    break Some(Ok(out_item));
+                                    Some(Ok(out_item))
                                 }
-                                Ok(None) => {
-                                    break None;
-                                }
-                                Err(err) => break Some(Err(err)),
+                                Ok(None) => None,
+                                Err(err) => Some(Err(err)),
                             }
                         }
-                        Ready(Some(Err(err))) => break Some(Err(err)),
-                        Ready(None) => break None,
+                        Ready(Some(Err(err))) => Some(Err(err)),
+                        Ready(None) => None,
                         Pending => {
                             *this.state = Some(state);
                             return Pending;
                         }
                     }
                 } else {
-                    break None;
+                    None
                 }
             })
         }
@@ -424,174 +420,176 @@ mod catch_error {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::utils::async_test;
 
-    #[tokio::test]
-    async fn take_until_error_test() {
-        {
-            let vec: Vec<Result<(), ()>> = stream::empty().take_until_error().collect().await;
-            assert_eq!(vec, []);
+    async_test! {
+        async fn take_until_error_test() {
+            {
+                let vec: Vec<Result<(), ()>> = stream::empty().take_until_error().collect().await;
+                assert_eq!(vec, []);
+            }
+
+            {
+                let vec: Vec<Result<_, ()>> = stream::iter([Ok(0), Ok(1), Ok(2), Ok(3)])
+                    .take_until_error()
+                    .collect()
+                    .await;
+                assert_eq!(vec, [Ok(0), Ok(1), Ok(2), Ok(3)]);
+            }
+
+            {
+                let vec: Vec<Result<_, _>> = stream::iter([Ok(0), Ok(1), Err(2), Ok(3)])
+                    .take_until_error()
+                    .collect()
+                    .await;
+                assert_eq!(vec, [Ok(0), Ok(1), Err(2),]);
+            }
         }
 
-        {
-            let vec: Vec<Result<_, ()>> = stream::iter([Ok(0), Ok(1), Ok(2), Ok(3)])
-                .take_until_error()
-                .collect()
-                .await;
-            assert_eq!(vec, [Ok(0), Ok(1), Ok(2), Ok(3)]);
-        }
 
-        {
-            let vec: Vec<Result<_, _>> = stream::iter([Ok(0), Ok(1), Err(2), Ok(3)])
-                .take_until_error()
-                .collect()
-                .await;
-            assert_eq!(vec, [Ok(0), Ok(1), Err(2),]);
-        }
-    }
-
-    #[tokio::test]
-    async fn try_stateful_then_test() {
-        {
-            let values: Result<Vec<_>, ()> = stream::iter([Ok(3), Ok(1), Ok(4), Ok(1)])
-                .try_stateful_then(0, |acc, val| async move {
-                    let new_acc = acc + val;
-                    Ok(Some((new_acc, new_acc)))
-                })
-                .try_collect()
-                .await;
-
-            assert_eq!(values, Ok(vec![3, 4, 8, 9]));
-        }
-
-        {
-            let mut stream = stream::iter([Ok(3), Ok(1), Err(()), Ok(1)])
-                .try_stateful_then(0, |acc, val| async move {
-                    let new_acc = acc + val;
-                    Ok(Some((new_acc, new_acc)))
-                })
-                .boxed();
-
-            assert_eq!(stream.next().await, Some(Ok(3)));
-            assert_eq!(stream.next().await, Some(Ok(4)));
-            assert_eq!(stream.next().await, Some(Err(())));
-            assert_eq!(stream.next().await, None);
-        }
-
-        {
-            let mut stream = stream::iter([Ok(3), Ok(1), Ok(4), Ok(1), Err(())])
-                .try_stateful_then(0, |acc, val| async move {
-                    let new_acc = acc + val;
-                    if new_acc != 8 {
+        async fn try_stateful_then_test() {
+            {
+                let values: Result<Vec<_>, ()> = stream::iter([Ok(3), Ok(1), Ok(4), Ok(1)])
+                    .try_stateful_then(0, |acc, val| async move {
+                        let new_acc = acc + val;
                         Ok(Some((new_acc, new_acc)))
-                    } else {
-                        Err(())
-                    }
-                })
-                .boxed();
+                    })
+                    .try_collect()
+                    .await;
 
-            assert_eq!(stream.next().await, Some(Ok(3)));
-            assert_eq!(stream.next().await, Some(Ok(4)));
-            assert_eq!(stream.next().await, Some(Err(())));
-            assert_eq!(stream.next().await, None);
-        }
+                assert_eq!(values, Ok(vec![3, 4, 8, 9]));
+            }
 
-        {
-            let mut stream = stream::iter([Ok(3), Ok(1), Ok(4), Ok(1), Err(())])
-                .try_stateful_then(0, |acc, val| async move {
-                    let new_acc = acc + val;
-                    if new_acc != 8 {
+            {
+                let mut stream = stream::iter([Ok(3), Ok(1), Err(()), Ok(1)])
+                    .try_stateful_then(0, |acc, val| async move {
+                        let new_acc = acc + val;
                         Ok(Some((new_acc, new_acc)))
-                    } else {
-                        Ok(None)
-                    }
-                })
-                .boxed();
+                    })
+                    .boxed();
 
-            assert_eq!(stream.next().await, Some(Ok(3)));
-            assert_eq!(stream.next().await, Some(Ok(4)));
-            assert_eq!(stream.next().await, None);
-        }
-    }
+                assert_eq!(stream.next().await, Some(Ok(3)));
+                assert_eq!(stream.next().await, Some(Ok(4)));
+                assert_eq!(stream.next().await, Some(Err(())));
+                assert_eq!(stream.next().await, None);
+            }
 
-    #[tokio::test]
-    async fn catch_error_test() {
-        {
-            let (notify, stream) = stream::empty::<Result<(), ()>>().catch_error();
+            {
+                let mut stream = stream::iter([Ok(3), Ok(1), Ok(4), Ok(1), Err(())])
+                    .try_stateful_then(0, |acc, val| async move {
+                        let new_acc = acc + val;
+                        if new_acc != 8 {
+                            Ok(Some((new_acc, new_acc)))
+                        } else {
+                            Err(())
+                        }
+                    })
+                    .boxed();
 
-            let vec: Vec<_> = stream.collect().await;
-            let result = notify.await;
+                assert_eq!(stream.next().await, Some(Ok(3)));
+                assert_eq!(stream.next().await, Some(Ok(4)));
+                assert_eq!(stream.next().await, Some(Err(())));
+                assert_eq!(stream.next().await, None);
+            }
 
-            assert_eq!(vec, []);
-            assert_eq!(result, Ok(()));
+            {
+                let mut stream = stream::iter([Ok(3), Ok(1), Ok(4), Ok(1), Err(())])
+                    .try_stateful_then(0, |acc, val| async move {
+                        let new_acc = acc + val;
+                        if new_acc != 8 {
+                            Ok(Some((new_acc, new_acc)))
+                        } else {
+                            Ok(None)
+                        }
+                    })
+                    .boxed();
+
+                assert_eq!(stream.next().await, Some(Ok(3)));
+                assert_eq!(stream.next().await, Some(Ok(4)));
+                assert_eq!(stream.next().await, None);
+            }
         }
 
-        {
-            let (notify, stream) =
-                stream::iter([Result::<_, ()>::Ok(0), Ok(1), Ok(2), Ok(3)]).catch_error();
 
-            let vec: Vec<_> = stream.collect().await;
-            let result = notify.await;
+        async fn catch_error_test() {
+            {
+                let (notify, stream) = stream::empty::<Result<(), ()>>().catch_error();
 
-            assert_eq!(vec, [0, 1, 2, 3]);
-            assert_eq!(result, Ok(()));
-        }
+                let vec: Vec<_> = stream.collect().await;
+                let result = notify.await;
 
-        {
-            let (notify, stream) = stream::iter([Ok(0), Ok(1), Err(2), Ok(3)]).catch_error();
+                assert_eq!(vec, []);
+                assert_eq!(result, Ok(()));
+            }
 
-            let vec: Vec<_> = stream.collect().await;
-            let result = notify.await;
+            {
+                let (notify, stream) =
+                    stream::iter([Result::<_, ()>::Ok(0), Ok(1), Ok(2), Ok(3)]).catch_error();
 
-            assert_eq!(vec, [0, 1]);
-            assert_eq!(result, Err(2));
-        }
+                let vec: Vec<_> = stream.collect().await;
+                let result = notify.await;
 
-        {
-            let (notify, mut stream) = stream::empty::<Result<(), ()>>().catch_error();
+                assert_eq!(vec, [0, 1, 2, 3]);
+                assert_eq!(result, Ok(()));
+            }
 
-            let notify = match notify.try_catch() {
-                Continue(notify) => notify,
-                _ => unreachable!(),
-            };
+            {
+                let (notify, stream) = stream::iter([Ok(0), Ok(1), Err(2), Ok(3)]).catch_error();
 
-            assert_eq!(stream.next().await, None);
-            assert!(matches!(notify.try_catch(), Break(Ok(()))));
-        }
+                let vec: Vec<_> = stream.collect().await;
+                let result = notify.await;
 
-        {
-            let (notify, mut stream) = stream::iter([Result::<_, ()>::Ok(0)]).catch_error();
+                assert_eq!(vec, [0, 1]);
+                assert_eq!(result, Err(2));
+            }
 
-            let notify = match notify.try_catch() {
-                Continue(notify) => notify,
-                _ => unreachable!(),
-            };
+            {
+                let (notify, mut stream) = stream::empty::<Result<(), ()>>().catch_error();
 
-            assert_eq!(stream.next().await, Some(0));
-            let notify = match notify.try_catch() {
-                Continue(notify) => notify,
-                _ => unreachable!(),
-            };
+                let notify = match notify.try_catch() {
+                    Continue(notify) => notify,
+                    _ => unreachable!(),
+                };
 
-            assert_eq!(stream.next().await, None);
-            assert!(matches!(notify.try_catch(), Break(Ok(()))));
-        }
+                assert_eq!(stream.next().await, None);
+                assert!(matches!(notify.try_catch(), Break(Ok(()))));
+            }
 
-        {
-            let (notify, mut stream) = stream::iter([Ok(0), Err(2)]).catch_error();
+            {
+                let (notify, mut stream) = stream::iter([Result::<_, ()>::Ok(0)]).catch_error();
 
-            let notify = match notify.try_catch() {
-                Continue(notify) => notify,
-                _ => unreachable!(),
-            };
+                let notify = match notify.try_catch() {
+                    Continue(notify) => notify,
+                    _ => unreachable!(),
+                };
 
-            assert_eq!(stream.next().await, Some(0));
-            let notify = match notify.try_catch() {
-                Continue(notify) => notify,
-                _ => unreachable!(),
-            };
+                assert_eq!(stream.next().await, Some(0));
+                let notify = match notify.try_catch() {
+                    Continue(notify) => notify,
+                    _ => unreachable!(),
+                };
 
-            assert_eq!(stream.next().await, None);
-            assert!(matches!(notify.try_catch(), Break(Err(2))));
+                assert_eq!(stream.next().await, None);
+                assert!(matches!(notify.try_catch(), Break(Ok(()))));
+            }
+
+            {
+                let (notify, mut stream) = stream::iter([Ok(0), Err(2)]).catch_error();
+
+                let notify = match notify.try_catch() {
+                    Continue(notify) => notify,
+                    _ => unreachable!(),
+                };
+
+                assert_eq!(stream.next().await, Some(0));
+                let notify = match notify.try_catch() {
+                    Continue(notify) => notify,
+                    _ => unreachable!(),
+                };
+
+                assert_eq!(stream.next().await, None);
+                assert!(matches!(notify.try_catch(), Break(Err(2))));
+            }
         }
     }
 }
